%% This script demonstrates how to read the MPI08 data, namely
% the sequences, silhouettes, meshes, orientation data and projection matrices
% All the needed functions for this demo have been copied in the same
% m-file for portablity.
% To be able to execute the file seamlessly, download the MPI08 data from
% http://www.tnt.uni-hannover.de/project/MPI08_Database/
% Then, extract all .tar.gz files into one folder, and copy this script in
% the folder.
% 
% Please use matlab cell-mode commands: CTRL + DOWN/UP to jump to the next function/cell
%
% If there are any further questions please contact us at pons@tnt.uni-hannover.de
%
% This function is part of the MPI08 data set, to be obtained at 
% http://www.tnt.uni-hannover.de/project/MPI08_Database/
%
% Written by: Gerard Pons
% Revision:   1.0
% Date:       29.10.2010
%
% Copyright 2010, MPI Informatik, Germany, and Hannover University, Germany.
% All Rights Reserved Permission to use, copy, modify, and distribute this 
% software and its documentation for any non-commercial purpose is hereby 
% granted without fee, provided that the above copyright notice appear in 
% all copies and that both that copyright notice and this permission 
% notice appear in supporting documentation, and that the name of the 
% author not be used in advertising or publicity pertaining to distribution 
% of the software without specific, written prior permission. 
% THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY 
% PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER 
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF 
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
% CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% START OF DEMO   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Set the path of the inputFiles, silhouettes and orientation data
% Initialize the rigid motion and joint angles for this demo
RBM=[-0.653807 0.0054431 0.756642 1.41868 ;
    -0.0213255 0.999445 -0.0256169 0.808062
    -0.75636 -0.0328844 -0.653328 -0.327581
    0 0 0 1];
jointAngles=[0.0236756
    0.0676125
    -0.178778
    0.147858
    -0.0678317
    0.150878
    -0.335112
    0.160084
    0.165439
    0.264614
    0.190831
    0.260081
    0.0121725
    0.0235007
    -0.0078103
    -0.0910358
    -0.0413008
    0.0110857
    -0.0149712
    0.0135271
    -0.077753
    -0.226923];
close all
InputFilesDir=fullfile('./InputFiles');
SilhouettesDir=fullfile('./Silhouettes');
OrisDataDir=fullfile('./MPI08_PriorFiles');
% Specify actor and sequence
actor='hb';
seq='01_01';
%% First read the mesh and display it
figure('Name','3D mesh model')
disp('First read the mesh and display it ... ')
meshfile=fullfile(InputFilesDir,'Models',actor,[actor 'x_mesh.dat']);
[mesh]=readMesh(meshfile);
drawMesh(mesh)
pause
close(1)
%% Second read the sensor data and display it
figure('Name','Draw 3D mesh with the sensors')
disp('Reading sensor data ... ')
orisFile=fullfile(OrisDataDir,actor,seq,['SensorOrisTaSaOnlyGlobal_',actor,'_',seq,'.txt']);
[ SensorOris ] = read_sensors( orisFile);
% Initialize the mesh so that it fits the images in the first frame
disp('drawing mesh tohether with sensor orientations... ')
M = angleToMatrix(mesh, RBM , [0;jointAngles]);
mesh = rigidMotion(mesh, M);
drawMesh(mesh)
drawnow
hold on
frame=1;
h_sensors = drawSensorOris(SensorOris,mesh, frame);
drawnow
pause
hold off
close(1)
%% Third read and decode the silhouettes
figure('Name','Multiview Images')
disp('Reading and decoding silhouettes ... ')
nViews=8;
% Just show the first frame of the sequence
for fr=1:1
    for v=1:nViews
        view=sprintf('%02d',v-1);
        file_in=['MPI_' actor '_' seq '_040_' view '.mat'];
        file_in=fullfile(SilhouettesDir,actor,seq,file_in);
        silh=load(file_in);
        tImRegion(:,:,v)=reshape(runLengthDecode(silh.frameRLE{fr}),[silh.frameHeight silh.frameWidth]);
    end
    % display multiview
    multiView=[tImRegion(:,:,1) tImRegion(:,:,2) tImRegion(:,:,3);
        tImRegion(:,:,4) tImRegion(:,:,5) tImRegion(:,:,6);
        tImRegion(:,:,7) zeros(silh.frameHeight,silh.frameWidth) tImRegion(:,:,8)];
    imshow(multiView,[])
end
pause
close(1)
%% Fourth display the projected mesh
figure('Name','Projected mesh plot')
disp('Projected mesh plot ... ')
nViews=8;
h_parent=gca;
for v=1:nViews
    view=sprintf('%02d',v-1);
    proj_path=fullfile(InputFilesDir,'ProjectionMatrices');
    P  = readProjMat( proj_path, v-1 );
    imshow(tImRegion(:,:,v));
    hold on
    drawProjectedMesh(mesh,P,h_parent);
    drawnow
    disp(['Current view ' view '  ... Press ENTER to see the next view'])
    pause
    hold off
end
disp('')
disp('Thanks for using MPI08 Database!')
disp('any further questions please contact pons@tnt.uni-hannover.de')
