function R = quat2matrix(Q)
%% This function converts a quaternion into homogenous matrix
% R = quat2matrix(Q)
% Converts quaternions to the corresponding rotation matrix
% representations.
%
% Input:    Q, 4xn array of quaterions
%
% Output:   R, 3x3xn array of rotation matrices
%
% Reference: Shoemake(1985)
%
% Remark:   Shoemake uses the quaternion rotation formula x' = q^{-1}xq.
%           We use the more common convention x' = qxq^{-1}. This leads to
%           transposed matrices (opposed to Shoemake's formulas), or,
%           equivalently, inverted quaternion representations.
%
% This function is part of the MPI08 data set, to be obtained at 
% http://www.tnt.uni-hannover.de/project/MPI08_Database/
%
% Revision:   1.0
% Date:       01.10.2010
%
% Copyright 2010, MPI Informatik, Germany, and Hannover University, Germany.
% All Rights Reserved Permission to use, copy, modify, and distribute this
% software and its documentation for any non-commercial purpose is hereby
% granted without fee, provided that the above copyright notice appear in
% all copies and that both that copyright notice and this permission
% notice appear in supporting documentation, and that the name of the
% author not be used in advertising or publicity pertaining to distribution
% of the software without specific, written prior permission.
% THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
% CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.numPoints = size(X, 2);

n = size(Q,2);
nsq = sum(Q.^2);
P = Q./repmat(sqrt(nsq),4,1);
R = zeros(3,3,n);
w = Q(1,:);
x = Q(2,:);
y = Q(3,:);
z = Q(4,:);
R(1,1,:) = 1 - 2*y.^2 - 2*z.^2;
R(2,1,:) = 2*x.*y + 2*w.*z;
R(3,1,:) = 2*x.*z - 2*w.*y;
R(1,2,:) = 2*x.*y - 2*w.*z;
R(2,2,:) = 1 - 2*x.^2 - 2*z.^2;
R(3,2,:) = 2*y.*z + 2*w.*x;
R(1,3,:) = 2*x.*z + 2*w.*y;
R(2,3,:) = 2*y.*z - 2*w.*x;
R(3,3,:) = 1 - 2*x.^2 - 2*y.^2;
end
